; docformat = 'rst'
;+
; This is an example program to demonstrate how to create data to be displayed as an image
; on Google Earth with Coyote Graphics routines.
;
; :Categories:
;    Graphics
;    
; :Examples:
;    Save the program as "google_earth_image.pro" and run it like this::
;       IDL> .RUN google_earth_image
;       
;    Once you have done this, open Google Earth and add the google_earth_image.kml file
;    to it. If you move the google_earth_image.kml file, be sure to also move the 
;    google_earth_image1.png and google_earth_image3.png files that are created with it.
;    Keep all three files together.
;       
; :Author:
;    FANNING SOFTWARE CONSULTING::
;       David W. Fanning 
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: david@idlcoyote.com
;       Coyote's Guide to IDL Programming: http://www.idlcoyote.com
;
; :History:
;     Change History::
;        Written, 20 February 2013 by David W. Fanning.
;
; :Copyright:
;     Copyright (c) 2013, Fanning Software Consulting, Inc.
;-
PRO Google_Earth_Image
 
   ; Open and read the precipitation data file.
   filename = 'ST4.2005010112.24h.bin'       
   OPENR, lun, filename, /Get_Lun
   precip = FltArr(1121, 881)
   ReadU, lun, precip
   Free_Lun, lun
   
   ; Set up 15 colors, roughly equvilent to NOAA precipitation colors.
   ; Use black for missing values.
   colors = ['dark green', 'lime green', 'light sea green', 'yellow', 'khaki', $
       'dark goldenrod', 'light salmon', 'orange', 'red', 'sienna', 'purple', $
       'orchid', 'thistle', 'sky blue', 'black']
   TVLCT, cgColor(colors, /Triple), 1
   TVLCT, r, g, b, /Get
   palette = [ [r], [g], [b] ]
   
   ; Process precip data. Missing value = 9.999e20.
   s = Size(precip, /Dimensions)
   scaledPrecip = BytArr(s[0],s[1])
   levels = [0, 2, 5, 10, 15, 20, 25, 50, 75, 100, 125, 150, 200, 250]
   FOR j=1,13 DO BEGIN
       index = Where(precip GE levels[j-1] AND precip LT levels[j], count)
       IF count GT 0 THEN scaledPrecip[index] = Byte(j)
   ENDFOR
   index = Where(precip GT 250 AND precip LT 9.999e20, count)
   IF count GT 0 THEN scaledPrecip[index] = 14B
   missing = Where(precip EQ 9.999e20, missing_count)
   IF missing_count GT 0 THEN scaledPrecip[missing] = 15B ; Will be black.
   
   ; Set up the map structure. This image is a stereographic image. Prior to IDL 8, you
   ; had to use incorrect keywords to set this map projection up.
   IF Float(!Version.Release) LT 8 THEN BEGIN
      mapCoord = Obj_New('cgMap', "Polar Stereographic", CENTER_LONGITUDE=-105, CENTER_LATITUDE=70)
   ENDIF ELSE BEGIN
     mapCoord = Obj_New('cgMap', "Polar Stereographic", CENTER_LONGITUDE=-105, TRUE_SCALE_LATITUDE=70)
   ENDELSE
   
   ; Set up latitutes and longitudes at the corners of the image. (ll, ul, ur, lr)
   longitude = [-119.023D, -134.039, -59.959, -80.7500]
   latitude =  [  23.117D,   53.509,  45.619,  19.8057]
   
   ; Project those lat/lon points into UV space.
   uv = mapCoord -> Forward(longitude, latitude)
   
   ; To set up map projection space, we need values at left, top, right, and bottom
   ; of image. We calculate these in UV space. Note that these values are in the
   ; center of the pixel. We have to move them to the edge of the pixel below.
   ; The U direction corresponds to longitude, and the V direction to latitude.
   topv =   (uv[1,1] + uv[1,2]) * 0.5
   botv =   (uv[1,0] + uv[1,3]) * 0.5
   leftu =  (uv[0,0] + uv[0,1]) * 0.5
   rightu = (uv[0,2] + uv[0,3]) * 0.5
   
   ; Calculate the scales in UV directions. The variable s contains the image dimensions.
   xscale = (rightu-leftu) / (s[0]-1)
   yscale = (topv-botv) / (s[1]-1)
   
   ; Get the range of the UV box that envelopes the image.
   xrange = [uv[0,1] - (0.5 * xscale), uv[0,2] + (0.5 * xscale)]
   yrange = [uv[1,0] - (0.5 * yscale), uv[1,1] + (0.5 * yscale)]

   ; Update the map coordinate object.
   mapCoord -> SetProperty, XRANGE=xrange, YRANGE=yrange, POSITION=pos

   ; Add the precip data to the KML file. Make it 30% transparent.
   kmlFile = Obj_New('cgKML_File', 'google_earth_image.kml')
   cgImage2KML, scaledPrecip, mapCoord, Palette=palette, Missing_Value=15, $
       Description='24-Hour Precipition', PlaceName='Precipitation', $
       FlyTo=[-98,39,11000], AddToFile=kmlFile, Transparent=30
       
   ; Add a colorbar to the KML file.
   cgCBar2KML, NColors=13, Bottom=1, Divisions=14, Title='24 Hour Precipitation (mm)', $
       /Discrete, OOB_High=14, TickNames = StrTrim(levels,2), Charsize=1.5, $
       Description='Precipitation Values', $
       PlaceName='Precipitation Color Bar', $
       AddToFile=kmlFile, Width=400
       
   ; Save the KML file.
   kmlFile -> Save, /KMZ
   
   ; Destroy the KML object.
   kmlFile -> Destroy
   
END ;*****************************************************************

; This main program shows how to call the program and add it to Google Earth.

  ; Run the program to create the Google Earth KML file.
  Google_Earth_Image
  
 ; Open Google Earth and open the "google_earth_image.kml" file.
 
END